'use client';

import React, { FC, useEffect, useRef, useState } from 'react';
import { Modal, Button, message, Input } from 'antd';
import { CloseOutlined, StarFilled } from '@ant-design/icons';
import { i18nText } from '../../i18nKeys';
import { useLocalStorage } from '../../hooks/useLocalStorage';
import { AUTH_TOKEN_KEY } from '../../constants/common';
import { getReport, saveReport } from '../../actions/rooms';
import { Report, ReportData } from '../../types/rooms';
import { CustomRate } from '../view/CustomRate';

type SupervisorReportModalProps = {
    open: boolean;
    handleCancel: () => void;
    locale: string;
    refresh: () => void;
    roomId: number;
};

export const SupervisorReportModal: FC<SupervisorReportModalProps> = ({
    open,
    handleCancel,
    locale,
    roomId,
    refresh
}) => {
    const [jwt] = useLocalStorage(AUTH_TOKEN_KEY, '');
    const [loading, setLoading] = useState<boolean>(false);
    const [report, setReport] = useState<Report[] | undefined>();
    const reasonRef = useRef<HTMLTextAreaElement>(null);

    useEffect(() => {
        getReport(locale, jwt, roomId)
            .then((data) => {
                setReport(data);
            })
            .catch(() => {
                message.error('Не удалось получить отчет');
            })
    }, []);

    const onSaveRate = () => {
        const result: ReportData = {
            sessionId: roomId,
            sessionSupervisorScores: report || [],
            supervisorComment: reasonRef?.current?.resizableTextArea?.textArea?.value || ''
        };

        setLoading(true);
        saveReport(locale, jwt, result)
            .then(() => {
                handleCancel();
                refresh();
            })
            .catch(() => {
                message.error('Не удалось сохранить отчет');
            })
            .finally(() => {
                setLoading(false);
            })
    }

    const onChangeRate = (val: number, id: number) => {
        setReport(report ? report.map((item) => {
            if (item.evaluationCriteriaId === id) {
                return {
                    ...item,
                    score: val
                };
            }

            return item;
        }) : undefined);
    }

    return (
        <Modal
            className="b-modal"
            open={open}
            title={undefined}
            onOk={undefined}
            onCancel={handleCancel}
            footer={false}
            width={498}
            closeIcon={<CloseOutlined style={{ fontSize: 20, color: '#000' }}/>}
        >
            <div className="b-modal__report__content">
                <div className="b-rate-list">
                    {report && report.length > 0 && report.map(({ key, evaluationCriteriaId, score }) => (
                        <div key={evaluationCriteriaId} className="b-rate-list__item">
                            <div className="b-rate-list__item_title">{i18nText(`room.rating_${key?.toLowerCase()}`, locale)}</div>
                            <CustomRate
                                defaultValue={score || 0}
                                character={<StarFilled style={{ fontSize: 32 }} />}
                                onChange={(val: number) => onChangeRate(val, evaluationCriteriaId)}
                            />
                        </div>
                    ))}
                </div>
                <div>
                    <Input.TextArea
                        ref={reasonRef}
                        className="b-textarea"
                        rows={1}
                        placeholder={i18nText('room.tellAboutReason', locale)}
                    />
                </div>
                <div>
                    <Button
                        className="btn-apply"
                        onClick={onSaveRate}
                        loading={loading}
                    >
                        {i18nText('room.rate', locale)}
                    </Button>
                </div>
            </div>
        </Modal>
    );
};
